const pool = require("../config/database");
const { compareApiKeySecret } = require("../utils/apiKeyHelper");

async function apiAuthMiddleware(req, res, next) {
	const authHeader = req.headers.authorization;

	if (!authHeader || !authHeader.startsWith("Bearer ")) {
		return res
			.status(401)
			.json({
				error:
					"Unauthorized: Missing or malformed API key. Expected Bearer token.",
			});
	}

	const fullApiKey = authHeader.substring(7); // Remove "Bearer "
	const parts = fullApiKey.split("_");

	// Expects key format: prefix_identifierRandomPart_secretPart
	// So, identifier is parts[0] + '_' + parts[1]
	// And secret is parts[2]
	if (parts.length < 3) {
		// Basic check for fsk_random_secret format
		return res
			.status(401)
			.json({ error: "Unauthorized: Invalid API key format." });
	}

	// Reconstruct identifier: e.g., parts[0] = 'fsk', parts[1] = 'randompart' -> 'fsk_randompart'
	const apiKeyIdentifier = `${parts[0]}_${parts[1]}`;
	const providedSecret = parts.slice(2).join("_"); // secret part could contain underscores if generated differently, though unlikely with current helper

	if (!apiKeyIdentifier || !providedSecret) {
		return res
			.status(401)
			.json({ error: "Unauthorized: Invalid API key structure." });
	}

	try {
		const [apiKeyRecords] = await pool.query(
			"SELECT ak.id, ak.user_id, ak.hashed_api_key_secret, ak.expires_at, u.is_active as user_is_active, u.role as user_role FROM api_keys ak JOIN users u ON ak.user_id = u.id WHERE ak.api_key_identifier = ?",
			[apiKeyIdentifier]
		);

		if (apiKeyRecords.length === 0) {
			return res.status(401).json({ error: "Unauthorized: Invalid API key." });
		}

		const apiKeyRecord = apiKeyRecords[0];

		if (!apiKeyRecord.user_is_active) {
			return res
				.status(403)
				.json({ error: "Forbidden: User account is inactive." });
		}

		// Check for expiration (if implemented and expires_at is not null)
		if (
			apiKeyRecord.expires_at &&
			new Date(apiKeyRecord.expires_at) < new Date()
		) {
			return res.status(403).json({ error: "Forbidden: API key has expired." });
		}

		const isValid = await compareApiKeySecret(
			providedSecret,
			apiKeyRecord.hashed_api_key_secret
		);

		if (!isValid) {
			return res.status(401).json({ error: "Unauthorized: Invalid API key." });
		}

		// Attach user information and API key ID to request for use in controllers/routes
		req.user = {
			id: apiKeyRecord.user_id,
			role: apiKeyRecord.user_role, // Add other relevant user fields if needed
			// Potentially add more fields from the user table if fetched in the JOIN
		};
		req.apiKeyId = apiKeyRecord.id;

		// Update last_used_at (fire and forget, no need to await or block)
		pool
			.query(
				"UPDATE api_keys SET last_used_at = CURRENT_TIMESTAMP WHERE id = ?",
				[apiKeyRecord.id]
			)
			.catch((err) =>
				console.error("Failed to update API key last_used_at:", err)
			);

		next();
	} catch (error) {
		console.error("API Authentication error:", error);
		return res
			.status(500)
			.json({ error: "Internal Server Error during API authentication." });
	}
}

module.exports = apiAuthMiddleware;
