const passport = require("../config/passport");
const jwtService = require("../services/jwtService");
const rateLimit = require("express-rate-limit");

// JWT Authentication middleware
const authenticateJWT = (req, res, next) => {
	passport.authenticate("jwt", { session: false }, (err, user, info) => {
		if (err) {
			return res.status(500).json({
				success: false,
				message: "Authentication error",
				error: err.message,
			});
		}

		if (!user) {
			return res.status(401).json({
				success: false,
				message: info?.message || "Authentication required",
			});
		}

		req.user = user;
		next();
	})(req, res, next);
};

// Optional JWT Authentication (doesn't fail if no token)
const authenticateJWTOptional = (req, res, next) => {
	const authHeader = req.headers.authorization;
	const token = jwtService.extractTokenFromHeader(authHeader);

	if (!token) {
		return next(); // No token provided, continue without user
	}

	passport.authenticate("jwt", { session: false }, (err, user, info) => {
		if (!err && user) {
			req.user = user;
		}
		// Continue regardless of authentication result
		next();
	})(req, res, next);
};

// Role-based authorization middleware
const requireRole = (roles) => {
	if (typeof roles === "string") {
		roles = [roles];
	}

	return (req, res, next) => {
		if (!req.user) {
			return res.status(401).json({
				success: false,
				message: "Authentication required",
			});
		}

		if (!roles.includes(req.user.role)) {
			return res.status(403).json({
				success: false,
				message: "Insufficient permissions",
			});
		}

		next();
	};
};

// Check if user is admin or super admin
const requireAdmin = requireRole(["admin", "super_admin"]);

// Check if user is super admin
const requireSuperAdmin = requireRole(["super_admin"]);

// Check if user owns the resource or is admin
const requireOwnershipOrAdmin = (getResourceUserId) => {
	return async (req, res, next) => {
		try {
			if (!req.user) {
				return res.status(401).json({
					success: false,
					message: "Authentication required",
				});
			}

			// Super admins can access everything
			if (req.user.role === "super_admin") {
				return next();
			}

			// Get the user ID that owns the resource
			const resourceUserId = await getResourceUserId(req);

			// Check if user owns the resource or is admin
			if (
				req.user.id === resourceUserId ||
				["admin", "super_admin"].includes(req.user.role)
			) {
				return next();
			}

			return res.status(403).json({
				success: false,
				message: "Access denied. You can only access your own resources.",
			});
		} catch (error) {
			return res.status(500).json({
				success: false,
				message: "Authorization error",
				error: error.message,
			});
		}
	};
};

// Check if account is verified
const requireVerifiedAccount = (req, res, next) => {
	if (!req.user) {
		return res.status(401).json({
			success: false,
			message: "Authentication required",
		});
	}

	// Super admins don't need verification
	if (req.user.role === "super_admin") {
		return next();
	}

	if (!req.user.is_verified) {
		return res.status(403).json({
			success: false,
			message: "Please verify your email address to access this resource",
			requiresVerification: true,
		});
	}

	next();
};

// Rate limiting middleware for authentication endpoints
const authRateLimit = rateLimit({
	windowMs: 15 * 60 * 1000, // 15 minutes
	max: 5, // Limit each IP to 5 requests per windowMs
	message: {
		success: false,
		message: "Too many authentication attempts, please try again later",
	},
	standardHeaders: true,
	legacyHeaders: false,
	keyGenerator: (req) => {
		// Use IP and email if available for more granular rate limiting
		return req.ip + (req.body?.email || "");
	},
});

// Rate limiting for password reset
const passwordResetRateLimit = rateLimit({
	windowMs: 60 * 60 * 1000, // 1 hour
	max: 3, // Limit each IP to 3 password reset attempts per hour
	message: {
		success: false,
		message: "Too many password reset attempts, please try again later",
	},
	standardHeaders: true,
	legacyHeaders: false,
	keyGenerator: (req) => {
		return req.ip + (req.body?.email || "");
	},
});

// Rate limiting for registration
const registrationRateLimit = rateLimit({
	windowMs: 60 * 60 * 1000, // 1 hour
	max: 3, // Limit each IP to 3 registrations per hour
	message: {
		success: false,
		message: "Too many registration attempts, please try again later",
	},
	standardHeaders: true,
	legacyHeaders: false,
	keyGenerator: (req) => {
		return req.ip;
	},
});

// Middleware to check if user is active
const requireActiveAccount = (req, res, next) => {
	if (!req.user) {
		return res.status(401).json({
			success: false,
			message: "Authentication required",
		});
	}

	if (!req.user.is_active) {
		return res.status(403).json({
			success: false,
			message: "Your account has been deactivated. Please contact support.",
		});
	}

	next();
};

// Combine common authentication checks
const requireAuth = [authenticateJWT, requireActiveAccount];
const requireVerifiedAuth = [
	authenticateJWT,
	requireActiveAccount,
	requireVerifiedAccount,
];

// Legacy basic auth middleware (for backward compatibility during transition)
const basicAuth = require("basic-auth");

const httpAuthMiddleware = (req, res, next) => {
	if (!process.env.ADMIN_USER || !process.env.ADMIN_PASSWORD) {
		console.warn(
			"ADMIN_USER or ADMIN_PASSWORD not set. Admin routes are unprotected."
		);
		return next();
	}

	const user = basicAuth(req);
	if (
		!user ||
		user.name !== process.env.ADMIN_USER ||
		user.pass !== process.env.ADMIN_PASSWORD
	) {
		res.set("WWW-Authenticate", 'Basic realm="Admin Area"');
		return res.status(401).send("Authentication required.");
	}
	return next();
};

module.exports = {
	// JWT Authentication
	authenticateJWT,
	authenticateJWTOptional,

	// Authorization
	requireRole,
	requireAdmin,
	requireSuperAdmin,
	requireOwnershipOrAdmin,
	requireVerifiedAccount,
	requireActiveAccount,

	// Combined middleware
	requireAuth,
	requireVerifiedAuth,

	// Rate limiting
	authRateLimit,
	passwordResetRateLimit,
	registrationRateLimit,

	// Legacy (for backward compatibility)
	httpAuthMiddleware,
};
