const bcrypt = require("bcryptjs");
const crypto = require("crypto");
const { v4: uuidv4 } = require("uuid");
const db = require("../config/database"); // db is now an instance of sqlite3.Database

class User {
	// Helper to run queries with promises
	static _run(query, params = []) {
		return new Promise((resolve, reject) => {
			db.run(query, params, function (err) {
				if (err) {
					reject(err);
				} else {
					resolve(this); // { lastID, changes }
				}
			});
		});
	}

	static _get(query, params = []) {
		return new Promise((resolve, reject) => {
			db.get(query, params, (err, row) => {
				if (err) {
					reject(err);
				} else {
					resolve(row);
				}
			});
		});
	}

	static _all(query, params = []) {
		return new Promise((resolve, reject) => {
			db.all(query, params, (err, rows) => {
				if (err) {
					reject(err);
				} else {
					resolve(rows);
				}
			});
		});
	}

	// Create a new user
	static async create(userData) {
		const {
			email,
			password,
			first_name,
			last_name,
			role = "user",
			is_verified = 0, // SQLite uses 0 for false
		} = userData;

		const saltRounds = 12;
		const password_hash = await bcrypt.hash(password, saltRounds);
		const verification_token = crypto.randomBytes(32).toString("hex");
		const uuid = uuidv4();

		const query = `
      INSERT INTO users (uuid, email, password_hash, first_name, last_name, role, is_verified, verification_token, created_at, updated_at)
      VALUES (?, ?, ?, ?, ?, ?, ?, ?, datetime('now'), datetime('now'))
    `;
		const values = [
			uuid,
			email,
			password_hash,
			first_name,
			last_name,
			role,
			is_verified,
			verification_token,
		];

		try {
			const result = await User._run(query, values);
			return {
				id: result.lastID,
				uuid,
				email,
				first_name,
				last_name,
				role,
				is_verified,
				verification_token,
			};
		} catch (error) {
			if (error.message && error.message.includes("UNIQUE constraint failed")) {
				// Check for specific constraint if possible, e.g., error.message.includes("users.email")
				throw new Error("Email already exists");
			}
			throw error;
		}
	}

	// Find user by email
	static async findByEmail(email) {
		const query = "SELECT * FROM users WHERE email = ? AND is_active = 1";
		return User._get(query, [email]);
	}

	// Find user by ID
	static async findById(id) {
		const query = "SELECT * FROM users WHERE id = ? AND is_active = 1";
		return User._get(query, [id]);
	}

	// Find user by UUID
	static async findByUuid(uuid) {
		const query = "SELECT * FROM users WHERE uuid = ? AND is_active = 1";
		return User._get(query, [uuid]);
	}

	// Find user by verification token
	static async findByVerificationToken(token) {
		const query = "SELECT * FROM users WHERE verification_token = ?";
		return User._get(query, [token]);
	}

	// Find user by password reset token
	static async findByPasswordResetToken(token) {
		const query = `
      SELECT * FROM users 
      WHERE password_reset_token = ? 
      AND password_reset_expires > datetime('now')
      AND is_active = 1
    `;
		return User._get(query, [token]);
	}

	// Verify email
	static async verifyEmail(token) {
		const query = `
      UPDATE users 
      SET is_verified = 1, verification_token = NULL, updated_at = datetime('now')
      WHERE verification_token = ?
    `;
		const result = await User._run(query, [token]);
		return result.changes > 0;
	}

	// Update password
	static async updatePassword(id, newPassword) {
		const saltRounds = 12;
		const password_hash = await bcrypt.hash(newPassword, saltRounds);
		const query = `
      UPDATE users 
      SET password_hash = ?, password_reset_token = NULL, password_reset_expires = NULL, updated_at = datetime('now')
      WHERE id = ?
    `;
		const result = await User._run(query, [password_hash, id]);
		return result.changes > 0;
	}

	// Update password and clear must_change_password flag
	static async updatePasswordAndClearChangeFlag(id, newPassword) {
		const saltRounds = 12;
		const password_hash = await bcrypt.hash(newPassword, saltRounds);
		const query = `
      UPDATE users 
      SET password_hash = ?, 
          must_change_password = 0, 
          password_reset_token = NULL, 
          password_reset_expires = NULL, 
          updated_at = datetime('now')
      WHERE id = ?
    `;
		const result = await User._run(query, [password_hash, id]);
		return result.changes > 0;
	}

	// Set password reset token
	static async setPasswordResetToken(email) {
		const token = crypto.randomBytes(32).toString("hex");
		// SQLite expects DATETIME strings, ISO 8601 format is good
		const expires = new Date(Date.now() + 3600000).toISOString();

		const query = `
      UPDATE users 
      SET password_reset_token = ?, password_reset_expires = ?, updated_at = datetime('now')
      WHERE email = ? AND is_active = 1
    `;
		const result = await User._run(query, [token, expires, email]);
		if (result.changes > 0) {
			return { token, expires };
		}
		return null;
	}

	// Increment failed login attempts
	static async incrementFailedLoginAttempts(id) {
		// Note: SQLite's CASE WHEN THEN ELSE END syntax is similar to MySQL
		// Locking for 30 minutes
		const query = `
      UPDATE users 
      SET failed_login_attempts = failed_login_attempts + 1,
          account_locked_until = CASE 
            WHEN failed_login_attempts >= 4 THEN datetime('now', '+30 minutes')
            ELSE account_locked_until
          END,
          updated_at = datetime('now')
      WHERE id = ?
    `;
		await User._run(query, [id]);
	}

	// Reset failed login attempts
	static async resetFailedLoginAttempts(id) {
		const query = `
      UPDATE users 
      SET failed_login_attempts = 0, account_locked_until = NULL, updated_at = datetime('now')
      WHERE id = ?
    `;
		await User._run(query, [id]);
	}

	// Update last login
	static async updateLastLogin(id) {
		const query =
			"UPDATE users SET last_login = datetime('now'), updated_at = datetime('now') WHERE id = ?";
		await User._run(query, [id]);
	}

	// Deactivate user account
	static async deactivateUser(id) {
		const query =
			"UPDATE users SET is_active = 0, updated_at = datetime('now') WHERE id = ?";
		const result = await User._run(query, [id]);
		return result.changes > 0;
	}

	// Activate user account
	static async activateUser(id) {
		const query =
			"UPDATE users SET is_active = 1, updated_at = datetime('now') WHERE id = ?";
		const result = await User._run(query, [id]);
		return result.changes > 0;
	}

	// Update user profile
	static async updateProfile(id, updates) {
		const allowedFields = ["first_name", "last_name", "email"];
		const fieldsToUpdate = [];
		const values = [];

		for (const [key, value] of Object.entries(updates)) {
			if (allowedFields.includes(key) && value !== undefined) {
				fieldsToUpdate.push(`\`${key}\` = ?`); // Use backticks for field names just in case
				values.push(value);
			}
		}

		if (fieldsToUpdate.length === 0) {
			throw new Error("No valid fields to update");
		}

		values.push(id); // for the WHERE clause
		const query = `UPDATE users SET ${fieldsToUpdate.join(
			", "
		)}, updated_at = datetime('now') WHERE id = ?`;

		try {
			const result = await User._run(query, values);
			return result.changes > 0;
		} catch (error) {
			if (error.message && error.message.includes("UNIQUE constraint failed")) {
				// Check for specific constraint if possible, e.g., error.message.includes("users.email")
				throw new Error("Email already exists");
			}
			throw error;
		}
	}

	// Session management for JWT tokens
	static async saveSession(
		userId,
		tokenJti,
		expiresAt, // Should be an ISO string or Unix timestamp
		userAgent = null,
		ipAddress = null
	) {
		const query = `
      INSERT INTO user_sessions (user_id, token_jti, expires_at, user_agent, ip_address, created_at)
      VALUES (?, ?, ?, ?, ?, datetime('now'))
    `;
		// Ensure expiresAt is in a format SQLite understands (e.g., ISO string)
		const expiresAtFormatted = new Date(expiresAt).toISOString();
		const values = [userId, tokenJti, expiresAtFormatted, userAgent, ipAddress];
		const result = await User._run(query, values);
		return result.lastID;
	}

	static async isTokenBlacklisted(tokenJti) {
		const query =
			"SELECT 1 FROM user_sessions WHERE token_jti = ? AND expires_at > datetime('now')";
		const row = await User._get(query, [tokenJti]);
		return !!row; // True if a non-expired session with this JTI exists
	}

	static async revokeSession(tokenJti) {
		// Instead of deleting, we can mark as expired or delete. Deleting is simpler.
		const query = "DELETE FROM user_sessions WHERE token_jti = ?";
		const result = await User._run(query, [tokenJti]);
		return result.changes > 0;
	}

	static async revokeAllUserSessions(userId) {
		const query = "DELETE FROM user_sessions WHERE user_id = ?";
		const result = await User._run(query, [userId]);
		return result.changes > 0;
	}

	static async revokeAllUserSessionsExcept(userId, exceptJti) {
		const query =
			"DELETE FROM user_sessions WHERE user_id = ? AND token_jti != ?";
		const result = await User._run(query, [userId, exceptJti]);
		return result.changes > 0;
	}

	static async getUserActiveSessions(userId) {
		const query =
			"SELECT id, token_jti, user_agent, ip_address, created_at, expires_at FROM user_sessions WHERE user_id = ? AND expires_at > datetime('now') ORDER BY created_at DESC";
		return User._all(query, [userId]);
	}

	static async getSessionByJti(jti) {
		const query = "SELECT * FROM user_sessions WHERE token_jti = ?";
		return User._get(query, [jti]);
	}

	// Cleanup expired sessions (can be run periodically)
	static async cleanupExpiredSessions() {
		const query =
			"DELETE FROM user_sessions WHERE expires_at <= datetime('now')";
		const result = await User._run(query);
		console.log("Cleaned up " + result.changes + " expired sessions.");
		return result.changes;
	}

	// Get user statistics (example, adapt as needed)
	static async getUserStats(userId) {
		// This is a placeholder. You'll need to adjust based on actual needs and tables.
		// For example, count forms or submissions associated with the user.
		// const formsQuery = "SELECT COUNT(*) as form_count FROM forms WHERE user_id = ?";
		// const submissionsQuery = "SELECT COUNT(*) as submission_count FROM submissions WHERE user_id = ?";

		// const [formsResult] = await User._all(formsQuery, [userId]);
		// const [submissionsResult] = await User._all(submissionsQuery, [userId]);

		return {
			// form_count: formsResult ? formsResult.form_count : 0,
			// submission_count: submissionsResult ? submissionsResult.submission_count : 0,
			// Add other relevant stats
		};
	}

	// Find all users with pagination and filtering (example)
	static async findAll(page = 1, limit = 20, filters = {}) {
		let query =
			"SELECT id, uuid, email, first_name, last_name, role, is_verified, is_active, created_at, last_login FROM users";
		const queryParams = [];
		const whereClauses = [];

		if (filters.role) {
			whereClauses.push("role = ?");
			queryParams.push(filters.role);
		}
		if (filters.is_active !== undefined) {
			whereClauses.push("is_active = ?");
			queryParams.push(filters.is_active ? 1 : 0);
		}
		// Add more filters as needed

		if (whereClauses.length > 0) {
			query += " WHERE " + whereClauses.join(" AND ");
		}

		query += " ORDER BY created_at DESC LIMIT ? OFFSET ?";
		queryParams.push(limit, (page - 1) * limit);

		const users = await User._all(query, queryParams);

		// For total count, need a separate query without limit/offset
		let countQuery = "SELECT COUNT(*) as total FROM users";
		if (whereClauses.length > 0) {
			// Reuse queryParams for filters, but not for limit/offset
			const filterParams = queryParams.slice(0, whereClauses.length);
			countQuery += " WHERE " + whereClauses.join(" AND ");
			const countResult = await User._get(countQuery, filterParams);
			return { users, total: countResult.total, page, limit };
		} else {
			const countResult = await User._get(countQuery);
			return { users, total: countResult.total, page, limit };
		}
	}

	// Add other user methods as needed
}

module.exports = User;
