const express = require("express");
const pool = require("../config/database");
const apiAuthMiddleware = require("../middleware/apiAuthMiddleware");

const router = express.Router();

// All routes in this file will be protected by API key authentication
router.use(apiAuthMiddleware);

// GET /api/v1/forms - List forms for the authenticated user
router.get("/forms", async (req, res) => {
	try {
		const [forms] = await pool.query(
			`SELECT uuid, name, created_at, is_archived,
       (SELECT COUNT(*) FROM submissions WHERE form_uuid = f.uuid) as submission_count
       FROM forms f 
       WHERE f.user_id = ? 
       ORDER BY f.created_at DESC`,
			[req.user.id] // req.user.id is attached by apiAuthMiddleware
		);
		res.json({ success: true, forms });
	} catch (error) {
		console.error("API Error fetching forms for user:", req.user.id, error);
		res.status(500).json({ success: false, error: "Failed to fetch forms." });
	}
});

// GET /api/v1/forms/:formUuid/submissions - List submissions for a specific form
router.get("/forms/:formUuid/submissions", async (req, res) => {
	const { formUuid } = req.params;
	const page = parseInt(req.query.page) || 1;
	const limit = parseInt(req.query.limit) || 25; // Default 25 submissions per page for API
	const offset = (page - 1) * limit;

	try {
		// First, verify the user (from API key) owns the form
		const [formDetails] = await pool.query(
			"SELECT user_id, name FROM forms WHERE uuid = ?",
			[formUuid]
		);

		if (formDetails.length === 0) {
			return res.status(404).json({ success: false, error: "Form not found." });
		}

		if (formDetails[0].user_id !== req.user.id) {
			return res
				.status(403)
				.json({
					success: false,
					error: "Access denied. You do not own this form.",
				});
		}

		// Get total count of submissions for pagination
		const [countResult] = await pool.query(
			"SELECT COUNT(*) as total FROM submissions WHERE form_uuid = ?",
			[formUuid]
		);
		const totalSubmissions = countResult[0].total;
		const totalPages = Math.ceil(totalSubmissions / limit);

		// Fetch paginated submissions
		const [submissions] = await pool.query(
			"SELECT id, data, ip_address, submitted_at FROM submissions WHERE form_uuid = ? ORDER BY submitted_at DESC LIMIT ? OFFSET ?",
			[formUuid, limit, offset]
		);

		res.json({
			success: true,
			formName: formDetails[0].name,
			formUuid,
			pagination: {
				currentPage: page,
				totalPages: totalPages,
				totalSubmissions: totalSubmissions,
				limit: limit,
				perPage: limit, // Alias for limit
				count: submissions.length,
			},
			submissions,
		});
	} catch (error) {
		console.error(
			"API Error fetching submissions for form:",
			formUuid,
			"user:",
			req.user.id,
			error
		);
		res
			.status(500)
			.json({ success: false, error: "Failed to fetch submissions." });
	}
});

module.exports = router;
