const express = require("express");
const pool = require("../config/database");
const { sendNtfyNotification } = require("../services/notification");
const { sendSubmissionNotification } = require("../services/emailService");
const { verifyRecaptchaV2 } = require("../utils/recaptchaHelper");
const {
	createSubmissionRateLimiter,
	createFormSpecificRateLimiter,
	createStrictRateLimiter,
} = require("../middleware/redisRateLimiter");
const domainChecker = require("../middleware/domainChecker");

const router = express.Router();

// Initialize rate limiters
const submissionRateLimit = createSubmissionRateLimiter();
const formSpecificRateLimit = createFormSpecificRateLimiter();
const strictRateLimit = createStrictRateLimiter();

router.get("/health", (req, res) => res.status(200).json({ status: "ok" }));

router.post(
	"/submit/:formUuid",
	strictRateLimit, // First layer: strict per-IP rate limit across all forms
	submissionRateLimit, // Second layer: general submission rate limit per IP
	formSpecificRateLimit, // Third layer: specific form+IP rate limit
	domainChecker,
	async (req, res) => {
		const { formUuid } = req.params;
		const submissionData = { ...req.body };
		const ipAddress = req.ip;

		// Extract reCAPTCHA response from submission data
		const recaptchaToken = submissionData["g-recaptcha-response"];
		// Clean it from submissionData so it's not stored in DB or shown in notifications
		delete submissionData["g-recaptcha-response"];

		// Honeypot check (early exit)
		if (submissionData.honeypot_field && submissionData.honeypot_field !== "") {
			console.log(
				`Honeypot triggered for ${formUuid} by IP ${ipAddress}. Ignoring submission.`
			);
			if (submissionData._thankyou) {
				return res.redirect(submissionData._thankyou);
			}
			return res.send(
				"<h1>Thank You!</h1><p>Your submission has been received.</p>"
			);
		}

		// Fetch form settings first to check for reCAPTCHA status and other details
		let formSettings;
		try {
			const [forms] = await pool.query(
				"SELECT id, user_id, name, thank_you_url, thank_you_message, ntfy_enabled, is_archived, email_notifications_enabled, notification_email_address, recaptcha_enabled FROM forms WHERE uuid = ?",
				[formUuid]
			);
			if (forms.length === 0) {
				return res.status(404).send("Form endpoint not found.");
			}
			formSettings = forms[0];

			if (formSettings.is_archived) {
				return res
					.status(410)
					.send(
						"This form has been archived and is no longer accepting submissions."
					);
			}
		} catch (dbError) {
			console.error("Error fetching form settings during submission:", dbError);
			return res
				.status(500)
				.send("Error processing submission due to database issue.");
		}

		// Perform reCAPTCHA verification if it's enabled for this form
		if (formSettings.recaptcha_enabled) {
			if (!recaptchaToken) {
				console.warn(
					`reCAPTCHA enabled for form ${formUuid} but no token provided by IP ${ipAddress}.`
				);
				return res
					.status(403)
					.send(
						"reCAPTCHA is required for this form. Please complete the challenge."
					);
			}

			const isRecaptchaValid = await verifyRecaptchaV2(
				recaptchaToken,
				ipAddress
			);
			if (!isRecaptchaValid) {
				console.warn(
					`reCAPTCHA verification failed for form ${formUuid} from IP ${ipAddress}.`
				);
				return res
					.status(403)
					.send("reCAPTCHA verification failed. Please try again.");
			}
		} // If reCAPTCHA is not enabled, or if it was enabled and passed, proceed.

		// Main submission processing logic (moved DB query for form details up)
		let formNameForNotification = formSettings.name || `Form ${formUuid}`;
		try {
			const ntfyEnabled = formSettings.ntfy_enabled;
			const formOwnerUserId = formSettings.user_id;

			// Prepare form object for email service
			const formForEmail = {
				name: formSettings.name,
				email_notifications_enabled: formSettings.email_notifications_enabled,
				notification_email_address: formSettings.notification_email_address,
			};

			// Fetch form owner's email for default notification recipient
			let ownerEmail = null;
			if (formOwnerUserId) {
				const [users] = await pool.query(
					"SELECT email FROM users WHERE id = ?",
					[formOwnerUserId]
				);
				if (users.length > 0) {
					ownerEmail = users[0].email;
				} else {
					console.warn(
						`Owner user with ID ${formOwnerUserId} not found for form ${formUuid}.`
					);
				}
			}

			await pool.query(
				"INSERT INTO submissions (form_uuid, user_id, data, ip_address) VALUES (?, ?, ?, ?)",
				[formUuid, formOwnerUserId, JSON.stringify(submissionData), ipAddress]
			);
			console.log(
				`Submission received for ${formUuid} (user: ${formOwnerUserId}):`,
				submissionData
			);

			const submissionSummary = Object.entries(submissionData)
				.filter(([key]) => key !== "_thankyou")
				.map(([key, value]) => `${key}: ${value}`)
				.join(", ");

			if (ntfyEnabled) {
				await sendNtfyNotification(
					`New Submission: ${formNameForNotification}`,
					`Data: ${
						submissionSummary || "No data fields"
					}\nFrom IP: ${ipAddress}`,
					"high",
					"incoming_form"
				);
			}

			// Send email notification
			if (ownerEmail) {
				// Only attempt if we have an owner email (even if custom one is set, good to have fallback context)
				sendSubmissionNotification(
					formForEmail,
					submissionData,
					ownerEmail
				).catch((err) =>
					console.error(
						"Failed to send submission email directly in route:",
						err
					)
				); // Log error but don't block response
			} else if (
				formForEmail.email_notifications_enabled &&
				!formForEmail.notification_email_address
			) {
				console.warn(
					`Email notification enabled for form ${formUuid} but owner email could not be determined and no custom address set.`
				);
			}

			if (formSettings.thank_you_url) {
				return res.redirect(formSettings.thank_you_url);
			}

			if (formSettings.thank_you_message) {
				// Basic HTML escaping for safety
				const safeMessage = formSettings.thank_you_message
					.replace(/&/g, "&amp;")
					.replace(/</g, "&lt;")
					.replace(/>/g, "&gt;")
					.replace(/"/g, "&quot;")
					.replace(/'/g, "&#039;");
				return res.send(safeMessage);
			}

			if (submissionData._thankyou) {
				return res.redirect(submissionData._thankyou);
			}

			res.send(
				'<h1>Thank You!</h1><p>Your submission has been received.</p><p><a href="/">Back to form manager</a></p>'
			);
		} catch (error) {
			console.error("Error processing submission:", error);
			await sendNtfyNotification(
				`Submission Error: ${formNameForNotification}`,
				`Failed to process submission for ${formUuid} from IP ${ipAddress}. Error: ${error.message}`,
				"max"
			);
			res.status(500).send("Error processing submission.");
		}
	}
);

module.exports = router;
