const nodemailer = require("nodemailer");
require("dotenv").config(); // Ensure environment variables are loaded
const { Resend } = require("resend");
const logger = require("../../config/logger"); // Adjust path as needed

const resendApiKey = process.env.RESEND_API_KEY;
const emailFromAddress = process.env.EMAIL_FROM_ADDRESS;

if (!resendApiKey) {
	logger.warn(
		"RESEND_API_KEY is not set. Email notifications will be disabled."
	);
}
if (!emailFromAddress) {
	logger.warn(
		"EMAIL_FROM_ADDRESS is not set. Email notifications may not work correctly."
	);
}

const resend = resendApiKey ? new Resend(resendApiKey) : null;

class EmailService {
	constructor() {
		this.transporter = null;
		this.init();
	}

	async init() {
		try {
			// Create reusable transporter object using the default SMTP transport
			this.transporter = nodemailer.createTransporter({
				host: process.env.SMTP_HOST,
				port: process.env.SMTP_PORT || 587,
				secure: process.env.SMTP_SECURE === "true", // true for 465, false for other ports
				auth: {
					user: process.env.SMTP_USER,
					pass: process.env.SMTP_PASS,
				},
			});

			// Verify connection configuration
			if (this.transporter && process.env.SMTP_HOST) {
				await this.transporter.verify();
				console.log("Email service initialized successfully");
			} else {
				console.warn(
					"Email service not configured. Set SMTP environment variables."
				);
			}
		} catch (error) {
			console.error("Email service initialization failed:", error.message);
			this.transporter = null;
		}
	}

	// Check if email service is available
	isAvailable() {
		return this.transporter !== null;
	}

	// Send verification email
	async sendVerificationEmail(to, firstName, verificationToken) {
		if (!this.isAvailable()) {
			console.warn("Email service not available. Verification email not sent.");
			return false;
		}

		const verificationUrl = `${process.env.APP_URL}/auth/verify-email?token=${verificationToken}`;

		const subject = "Verify Your Email Address - Formies";
		const html = this.getVerificationEmailTemplate(firstName, verificationUrl);

		try {
			await this.transporter.sendMail({
				from: `"Formies" <${
					process.env.SMTP_FROM_EMAIL || process.env.SMTP_USER
				}>`,
				to,
				subject,
				html,
			});

			console.log(`Verification email sent to ${to}`);
			return true;
		} catch (error) {
			console.error("Failed to send verification email:", error);
			return false;
		}
	}

	// Send password reset email
	async sendPasswordResetEmail(to, firstName, resetToken) {
		if (!this.isAvailable()) {
			console.warn(
				"Email service not available. Password reset email not sent."
			);
			return false;
		}

		const resetUrl = `${process.env.APP_URL}/auth/reset-password?token=${resetToken}`;

		const subject = "Password Reset Request - Formies";
		const html = this.getPasswordResetEmailTemplate(firstName, resetUrl);

		try {
			await this.transporter.sendMail({
				from: `"Formies" <${
					process.env.SMTP_FROM_EMAIL || process.env.SMTP_USER
				}>`,
				to,
				subject,
				html,
			});

			console.log(`Password reset email sent to ${to}`);
			return true;
		} catch (error) {
			console.error("Failed to send password reset email:", error);
			return false;
		}
	}

	// Send welcome email
	async sendWelcomeEmail(to, firstName) {
		if (!this.isAvailable()) {
			console.warn("Email service not available. Welcome email not sent.");
			return false;
		}

		const subject = "Welcome to Formies!";
		const html = this.getWelcomeEmailTemplate(firstName);

		try {
			await this.transporter.sendMail({
				from: `"Formies" <${
					process.env.SMTP_FROM_EMAIL || process.env.SMTP_USER
				}>`,
				to,
				subject,
				html,
			});

			console.log(`Welcome email sent to ${to}`);
			return true;
		} catch (error) {
			console.error("Failed to send welcome email:", error);
			return false;
		}
	}

	// Send password changed notification
	async sendPasswordChangedEmail(to, firstName) {
		if (!this.isAvailable()) {
			return false;
		}

		const subject = "Password Changed Successfully - Formies";
		const html = this.getPasswordChangedEmailTemplate(firstName);

		try {
			await this.transporter.sendMail({
				from: `"Formies" <${
					process.env.SMTP_FROM_EMAIL || process.env.SMTP_USER
				}>`,
				to,
				subject,
				html,
			});

			return true;
		} catch (error) {
			console.error("Failed to send password changed email:", error);
			return false;
		}
	}

	// Email templates
	getVerificationEmailTemplate(firstName, verificationUrl) {
		return `
      <!DOCTYPE html>
      <html>
      <head>
        <style>
          body { font-family: Arial, sans-serif; line-height: 1.6; color: #333; }
          .container { max-width: 600px; margin: 0 auto; padding: 20px; }
          .header { background: #007bff; color: white; padding: 20px; text-align: center; }
          .content { padding: 20px; background: #f9f9f9; }
          .button { 
            display: inline-block; 
            padding: 12px 30px; 
            background: #007bff; 
            color: white; 
            text-decoration: none; 
            border-radius: 5px; 
            margin: 20px 0;
          }
          .footer { padding: 20px; text-align: center; color: #666; font-size: 12px; }
        </style>
      </head>
      <body>
        <div class="container">
          <div class="header">
            <h1>Welcome to Formies!</h1>
          </div>
          <div class="content">
            <h2>Hi ${firstName || "there"},</h2>
            <p>Thank you for signing up for Formies! To complete your registration, please verify your email address by clicking the button below:</p>
            <p style="text-align: center;">
              <a href="${verificationUrl}" class="button">Verify Email Address</a>
            </p>
            <p>If the button doesn't work, you can copy and paste this link into your browser:</p>
            <p style="word-break: break-all; background: #eee; padding: 10px;">${verificationUrl}</p>
            <p><strong>This link will expire in 24 hours.</strong></p>
            <p>If you didn't create an account with Formies, you can safely ignore this email.</p>
          </div>
          <div class="footer">
            <p>© 2024 Formies. All rights reserved.</p>
          </div>
        </div>
      </body>
      </html>
    `;
	}

	getPasswordResetEmailTemplate(firstName, resetUrl) {
		return `
      <!DOCTYPE html>
      <html>
      <head>
        <style>
          body { font-family: Arial, sans-serif; line-height: 1.6; color: #333; }
          .container { max-width: 600px; margin: 0 auto; padding: 20px; }
          .header { background: #dc3545; color: white; padding: 20px; text-align: center; }
          .content { padding: 20px; background: #f9f9f9; }
          .button { 
            display: inline-block; 
            padding: 12px 30px; 
            background: #dc3545; 
            color: white; 
            text-decoration: none; 
            border-radius: 5px; 
            margin: 20px 0;
          }
          .footer { padding: 20px; text-align: center; color: #666; font-size: 12px; }
        </style>
      </head>
      <body>
        <div class="container">
          <div class="header">
            <h1>Password Reset Request</h1>
          </div>
          <div class="content">
            <h2>Hi ${firstName || "there"},</h2>
            <p>We received a request to reset your password for your Formies account. If you made this request, click the button below to reset your password:</p>
            <p style="text-align: center;">
              <a href="${resetUrl}" class="button">Reset Password</a>
            </p>
            <p>If the button doesn't work, you can copy and paste this link into your browser:</p>
            <p style="word-break: break-all; background: #eee; padding: 10px;">${resetUrl}</p>
            <p><strong>This link will expire in 1 hour.</strong></p>
            <p>If you didn't request a password reset, you can safely ignore this email. Your password won't be changed.</p>
          </div>
          <div class="footer">
            <p>© 2024 Formies. All rights reserved.</p>
          </div>
        </div>
      </body>
      </html>
    `;
	}

	getWelcomeEmailTemplate(firstName) {
		return `
      <!DOCTYPE html>
      <html>
      <head>
        <style>
          body { font-family: Arial, sans-serif; line-height: 1.6; color: #333; }
          .container { max-width: 600px; margin: 0 auto; padding: 20px; }
          .header { background: #28a745; color: white; padding: 20px; text-align: center; }
          .content { padding: 20px; background: #f9f9f9; }
          .button { 
            display: inline-block; 
            padding: 12px 30px; 
            background: #28a745; 
            color: white; 
            text-decoration: none; 
            border-radius: 5px; 
            margin: 20px 0;
          }
          .footer { padding: 20px; text-align: center; color: #666; font-size: 12px; }
        </style>
      </head>
      <body>
        <div class="container">
          <div class="header">
            <h1>Welcome to Formies!</h1>
          </div>
          <div class="content">
            <h2>Hi ${firstName || "there"},</h2>
            <p>Welcome to Formies! Your email has been verified and your account is now active.</p>
            <p>You can now start creating beautiful forms and collecting submissions. Here's what you can do:</p>
            <ul>
              <li>Create unlimited forms</li>
              <li>Customize form fields and styling</li>
              <li>Receive instant notifications</li>
              <li>Export your data anytime</li>
            </ul>
            <p style="text-align: center;">
              <a href="${
								process.env.APP_URL
							}/dashboard" class="button">Go to Dashboard</a>
            </p>
          </div>
          <div class="footer">
            <p>© 2024 Formies. All rights reserved.</p>
          </div>
        </div>
      </body>
      </html>
    `;
	}

	getPasswordChangedEmailTemplate(firstName) {
		return `
      <!DOCTYPE html>
      <html>
      <head>
        <style>
          body { font-family: Arial, sans-serif; line-height: 1.6; color: #333; }
          .container { max-width: 600px; margin: 0 auto; padding: 20px; }
          .header { background: #17a2b8; color: white; padding: 20px; text-align: center; }
          .content { padding: 20px; background: #f9f9f9; }
          .footer { padding: 20px; text-align: center; color: #666; font-size: 12px; }
        </style>
      </head>
      <body>
        <div class="container">
          <div class="header">
            <h1>Password Changed</h1>
          </div>
          <div class="content">
            <h2>Hi ${firstName || "there"},</h2>
            <p>This email confirms that your password has been successfully changed for your Formies account.</p>
            <p><strong>If you didn't make this change, please contact our support team immediately.</strong></p>
            <p>For your security, here are some tips:</p>
            <ul>
              <li>Use a strong, unique password</li>
              <li>Don't share your password with anyone</li>
              <li>Consider using a password manager</li>
            </ul>
          </div>
          <div class="footer">
            <p>© 2024 Formies. All rights reserved.</p>
          </div>
        </div>
      </body>
      </html>
    `;
	}
}

/**
 * Generates a simple HTML body for the submission notification email.
 * @param {string} formName - The name of the form.
 * @param {object} submissionData - The data submitted to the form.
 * @returns {string} - HTML string for the email body.
 */
function createEmailHtmlBody(formName, submissionData) {
	let body = `<p>You have a new submission for your form: <strong>${formName}</strong>.</p>`;
	body += "<p>Here are the details:</p><ul>";
	for (const [key, value] of Object.entries(submissionData)) {
		// Exclude honeypot and other internal fields if necessary
		if (key.toLowerCase() !== "honeypot_field" && key !== "_thankyou") {
			body += `<li><strong>${key}:</strong> ${value}</li>`;
		}
	}
	body += "</ul><p>Thank you for using Formies!</p>";
	return body;
}

/**
 * Sends a submission notification email.
 * @param {object} form - Form details (name, email_notifications_enabled, notification_email_address).
 * @param {object} submissionData - The actual data submitted to the form.
 * @param {string} userOwnerEmail - The email of the user who owns the form.
 */
async function sendSubmissionNotification(
	form,
	submissionData,
	userOwnerEmail
) {
	if (!resend) {
		logger.warn(
			"Resend SDK not initialized due to missing API key. Skipping email notification."
		);
		return;
	}
	if (!emailFromAddress) {
		logger.warn(
			"EMAIL_FROM_ADDRESS not configured. Skipping email notification."
		);
		return;
	}

	if (!form || !form.email_notifications_enabled) {
		logger.info(
			`Email notifications are disabled for form: ${form ? form.name : "Unknown Form"}. Skipping.`
		);
		return;
	}

	const recipientEmail = form.notification_email_address || userOwnerEmail;
	if (!recipientEmail) {
		logger.warn(
			`No recipient email address found for form: ${form.name}. Skipping notification.`
		);
		return;
	}

	const subject = `New Submission for Form: ${form.name}`;
	const htmlBody = createEmailHtmlBody(form.name, submissionData);

	try {
		const { data, error } = await resend.emails.send({
			from: emailFromAddress,
			to: recipientEmail,
			subject: subject,
			html: htmlBody,
		});

		if (error) {
			logger.error("Error sending submission email via Resend:", error);
			// Do not let email failure break the submission flow (as per 2.3.4)
			return; // Or throw a specific error to be caught upstream if needed for more complex handling
		}

		logger.info(
			`Submission email sent successfully to ${recipientEmail} for form ${form.name}. Message ID: ${data ? data.id : "N/A"}`
		);
	} catch (err) {
		logger.error("Exception caught while sending submission email:", err);
		// Do not let email failure break the submission flow
	}
}

module.exports = {
	sendSubmissionNotification,
	// Potentially export createEmailHtmlBody if it needs to be used elsewhere or for testing
};
